#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Efficient logger for the application with rich panel adjustments and fractional second display."""

from __future__ import annotations

from atexit import register
from pathlib import Path
from typing import Any, Dict, List, Optional, Tuple

import loguru
from dotenv import load_dotenv
from loguru import logger
from rich.console import Console
from rich.panel import Panel
from rich.progress import (
    BarColumn,
    MofNCompleteColumn,
    Progress,
    SpinnerColumn,
    TaskProgressColumn,
    TextColumn,
    TimeElapsedColumn,
    TimeRemainingColumn,
)
from rich.style import Style
from rich.text import Text
from rich_gradient import Color, Gradient

from ludden_logging.run import Run, console

HOME: Path = Path.home()
CWD: Path = Path.cwd()
LOGS_DIR: Path = CWD / "logs"

load_dotenv()


class Log:
    """Efficient logger for the application with rich panel adjustments and fractional second display."""

    FORMAT: str = "{time:HH:mm:ss.SSS} | Run {extra[run]} | {file.name: ^12} | Line {line} | {level} | {message}"

    LEVEL_STYLES: Dict[str, Style] = {
        "TRACE": Style(italic=True),
        "DEBUG": Style(color="#aaaaaa"),
        "INFO": Style(color="#00afff"),
        "SUCCESS": Style(bold=True, color="#00ff00"),
        "WARNING": Style(italic=True, color="#ffaf00"),
        "ERROR": Style(bold=True, color="#ff5000"),
        "CRITICAL": Style(bold=True, color="#ff0000"),
    }

    GRADIENTS: Dict[str, Tuple[List[Color], bool, bool]] = {
        "TRACE": ([Color("#888888"), Color("#aaaaaa"), Color("#cccccc")], True, False),
        "DEBUG": ([Color("#338888"), Color("#55aaaa"), Color("#77cccc")], False, False),
        "INFO": ([Color("#008fff"), Color("#00afff"), Color("#00cfff")], True, False),
        "SUCCESS": (
            [Color("#00aa00"), Color("#00ff00"), Color("#afff00")],
            True,
            False,
        ),
        "WARNING": (
            [Color("#ffaa00"), Color("#ffcc00"), Color("#ffff00")],
            True,
            False,
        ),
        "ERROR": ([Color("#ff0000"), Color("#ff5500"), Color("#ff7700")], True, False),
        "CRITICAL": (
            [Color("#ff0000"), Color("#ff005f"), Color("#ff00af")],
            True,
            True,
        ),
    }
    progress = Progress(
        TextColumn("[progress.description]{task.description}"),
        SpinnerColumn(),
        BarColumn(bar_width=None),
        MofNCompleteColumn(),
        TaskProgressColumn(),
        TimeElapsedColumn(),
        TimeRemainingColumn(),
        console=console,
    )

    def __init__(
        self,
        user_run: Optional[int] = None,
        console: Console = console,
        log_to_console: int = 20,
        progress: bool = False,
    ) -> None:
        """Initialize logger and configure sinks.

        Args:
            run (Any): The Run object to get the current run count.
            console (Console, optional): The console object for logging. Defaults to console.
            log_to_console (int, optional): The log level to display in the console. Defaults to 20.
            progress (bool, optional): Whether to display a progress bar. Defaults to False.
        """
        self.console: Console = console
        self.console.line(9)
        self.console.clear()

        # Deal with Run object or run count
        if not user_run:
            run_instance = Run()
            run = run_instance.load()
            self.run = run
        else:
            self.run = user_run
        logger.remove()
        self.log_file = self.verify_log_file()

        # Initialize logging sinks only once
        logger.add(
            self.log_file,
            format=self.FORMAT,
            level="TRACE",
            backtrace=True,
            enqueue=True,
            colorize=True,
        )
        logger.add(
            self.rich_sink,
            format="{message}",
            level=log_to_console,
            backtrace=True,
            enqueue=True,
        )

        # Store extra information
        logger.configure(extra={"run": self.run})

    @property
    def run(self) -> int:
        """Get the current run count."""
        return self._run

    @run.setter
    def run(self, run: Optional[int] = None) -> None:
        """Set the current run count."""
        if run is None:
            if self.run is None:
                self._run: int = 0
            else:
                self._run = self.run
        else:
            self._run = run

    def increment(self) -> int:
        """Increment the run count."""
        self.run = self.run + 1
        return self.run

    def verify_logs_dir(self) -> Path:
        """Verify the existence of the logs directory.

        Returns:
            the logs directory path.
        """
        CWD = Path.cwd()
        LOGS_DIR = CWD / "logs"
        if not LOGS_DIR.exists():
            LOGS_DIR.mkdir(parents=True)
        return LOGS_DIR

    def verify_log_file(self) -> Path:
        """Get the log file path."""
        LOGS_DIR = self.verify_logs_dir()
        log_file = LOGS_DIR / "trace.log"
        return log_file

    def log(self, level: str | int, message: Any) -> None:
        """Log a message.

        Args:
            level (str): The logging level.
            message (Any): The object(s) to log.
        """
        if Path(__file__).parent.name == "ludden_logging":
            depth = 1
        else:
            depth = 2
        if isinstance(level, int):
            logger.opt(depth=depth).log(
                level,
                message,
            )
        else:
            logger.opt(depth=depth).log(level.upper(), message)

    def trace(self, message: Any) -> None:
        """Log a message with level 'TRACE'.

        Args:
            message (Any): The object(s) to log.

        Example:
            >>> log.trace("Trace message.")
        """
        logger.trace(message)

    def debug(self, message: Any) -> None:
        """Log a message with the level 'DEBUG'.

        Args:
            message (Any): The object(s) to log.

        Example:
            >>> log.debug("Debug message.")
        """
        logger.debug(message)

    def info(self, message: Any) -> None:
        """Log a message with the level 'INFO'.

        Args:
            message (Any): The object(s) to log.

        Example:
            >>> log.info("Info message.")
        """
        logger.info(message)

    def success(self, message: Any) -> None:
        """Log a message with the level 'SUCCESS'.

        Args:
            message (Any): The object(s) to log.

        Example:
            >>> log.success("Success message.")
        """
        logger.success(message)

    def warning(self, message: Any) -> None:
        """Log a message with the level 'WARNING'.

        Args:
            message (Any): The object(s) to log.

        Example:
            >>> log.warning("Warning message.")
        """
        logger.warning(message)

    def error(self, message: Any) -> None:
        """Log an message with the level 'ERROR'.

        Args:
            message (Any): The object(s) to log.

        Example:
            >>> log.error("Error message.")
        """
        logger.error(message)

    def critical(self, message: Any) -> None:
        """Log a message with the level 'CRITICAL'.

        Args:
            message (Any): The object(s) to log.

        Example:
            >>> log.critical("Critical message.")
        """
        logger.critical(message)

    def rich_sink(self, message: Any) -> None:
        """Custom Rich sink to log styled messages to the console."""
        record: loguru.Record = message.record
        level: str = record["level"].name
        style: Style = self.LEVEL_STYLES.get(level, Style())

        colors, bold, italic = self.GRADIENTS.get(level, ([], False, False))

        # Title includes log level and file/line info
        title: Text = Gradient(
            f" {level} | {record['file'].name} | Line {record['line']} ", colors
        ).as_text()
        title.highlight_words("|", style="italic #666666")
        title.stylize(Style(reverse=True))

        # Subtitle includes run count and time with fractional seconds (milliseconds)
        subtitle: Text = Text.assemble(
            Text(f"Run {self.run} | "),
            Text(record["time"].strftime("%H:%M:%S.%f")[:-3]),
            Text(record["time"].strftime(" %p")),
        )
        subtitle.highlight_words(":", style="dim #aaaaaa")

        # Message text
        message_text: Text = Gradient(record["message"], colors, style="bold")

        # Generate and print log panel with aligned title and subtitle
        log_panel: Panel = Panel(
            message_text,
            title=title,
            title_align="left",  # Left align the title
            subtitle=subtitle,
            subtitle_align="right",  # Right align the subtitle
            border_style=style + Style(bold=True),
            padding=(1, 2),
        )
        self.console.print(log_panel)

    def opt(
        self,
        *,
        exception: Optional[bool | Tuple | Exception] = None,
        record: bool = False,
        lazy: bool = False,
        colors: bool = False,
        raw: bool = False,
        capture: bool = True,
        depth: int = 0,
        ansi: int = False,
    ) -> loguru.Logger:
        """Set options for the logger.

        Args:
            exception (bool, tuple, Exception, optional):
                If It Does Not Evaluate As ``False``, The Passed Exception Is Formatted And Added To The
                Log Message. It Could Be An |Exception| Object Or A ``(Type, Value, Traceback)`` Tuple,
                Otherwise The Exception Information Is Retrieved From |Sys.Exc_info|. Defaults to None.
            record (bool, optional):
                If ``True``, The Record Dict Contextualizing The Logging Call Can Be Used To Format The
                Message By Using ``{Record[Key]}`` In The Log Message. Defaults to False.
            lazy (bool, optional):
                If ``True``, The Logging Call Attribute To Format The Message Should Be Functions Which
                Will Be Called Only If The Level Is High Enough. This Can Be Used To Avoid Expensive
                Functions If Not Necessary. Defaults to False.
            colors (bool, optional):
                If ``True``, Logged Message Will Be Colorized According To The Markups It Possibly
                Contains. Defaults to False.
            raw (bool, optional):
                If ``True``, The Formatting Of Each Sink Will Be Bypassed And The Message Will Be Sent
                As Is. Defaults to False.
            capture (bool, optional):
                If ``False``, The ``**Kwargs`` Of Logged Message Will Not Automatically Populate
                The ``Extra`` Dict (Although They Are Still Used For Formatting). Defaults to True.
            depth (int, optional):
                Specify Which Stacktrace Should Be Used To Contextualize The Logged Message. This Is
                Useful While Using The Logger From Inside A Wrapped Function To Retrieve Worthwhile
                Information. Defaults to 0.
            ansi (int, optional):
                Deprecated Since Version 0.4.1: The ``Ansi`` Parameter Will Be Removed In Loguru 1.0.0,
                It Is Replaced By ``Colors`` Which Is A More Appropriate Name. Defaults to False.
        """
        return logger.opt(
            exception=exception,  # type: ignore
            record=record,
            lazy=lazy,
            colors=colors,
            raw=raw,
            capture=capture,
            depth=depth,
            ansi=ansi,  # type: ignore
        )


register(Log().increment)  # type: ignore

if __name__ == "__main__":
    from ludden_logging.run import Run  # Assuming Run is defined in run.py

    log = Log()
    log.increment()

    # Log test messages
    log.log("TRACE", "Trace message.")
    log.log("DEBUG", "Debug message.")
    log.log("INFO", "Info message.")
    log.log("SUCCESS", "Success message.")
    log.log("WARNING", "Warning message.")
    log.log("ERROR", "Error message.")
    log.log("CRITICAL", "Critical message.")




from rich.console import Console
from rich.traceback import install as tr_install
# from atexit import register

from ludden_logging.run import Run
from ludden_logging.log import Log

__all__ = [
    "Log",
    "Run"
]

console = Console()
tr_install(console=console)
run = Run().load()
log = Log(run)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Get the current working directory as a Path object."""
from os import getenv
from pathlib import Path
from typing import Optional

from ludden_logging import console


def cwd(path: Optional[Path] = None, verbose: bool = False) -> Path:
    """Get the current working directory as a Path object."""
    console.clear()
    console.line(2)
    venv = getenv("VIRTUAL_ENV_PROMPT")
    cwd = Path.cwd()
    if verbose:
        console.log(f"Current virtual environment: {venv}")
        console.log(f"Current working directory: {cwd}")
        console.line(2)
    if not venv:
        console.log("No virtual environment detected.")
        raise EnvironmentError("No virtual environment detected.")
    else:
        if venv == cwd.parent.stem:
            if verbose:
                console.log("The virtual environment matches the parent directory.")
            return cwd
        else:
            return Path.home() / "dev" / "py" / venv


if __name__ == '__main__':
    cwd(verbose=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Track the number of runs of a module, stored in both a file and an environment variable."""
import os
from pathlib import Path
from typing import Optional

from rich.console import Console
from rich.prompt import IntPrompt
from rich.traceback import install as tr_install


# from ludden_logging import console

console = Console(
    # style=Style(color="#ffffff", bgcolor="#30173d", bold=True),
    # width=120,
)

tr_install(console=console)


class Run:
    """Class to track the number of runs of a module, stored in both a file and an environment variable."""

    white = "#ffffff"
    radiate = "#d7ff64"
    space = "#30173d"
    tab = "\t\t"

    def __init__(
        self,
        run: Optional[int] = None,
        project_name: Optional[str] = None,
        run_file: Optional[Path] = None,
        console: Console = console,
        verbose: bool = False,
    ) -> None:
        """Initialize the Run class and load the run count.

        Args:
            project_name (Optional[str], optional): The name of the project for setting up the environment variable.
            run_file (Optional[Path], optional): The file where the run count is stored. Defaults to 'logs/run.txt'.
        """
        # Set the project name
        self.project_name = project_name or self.get_project_name()

        # Set the verbose flag
        self.verbose: bool = verbose
        if verbose:
            # Print the project name
            console.line()
            console.print(
                f"[b on {self.space}][#ffffff]Project name:[/#ffffff] \
[{self.radiate}]{self.project_name}[/{self.radiate}]"
            )

        # Set the environment variable name
        self.env_var_name = f"{self.project_name.upper()}_RUN_COUNT"

        # Default log file location
        self.run_file = run_file or Path.cwd() / "logs" / "run.txt"
        self.run_file.parent.mkdir(
            parents=True, exist_ok=True
        )  # Ensure the log directory exists

        self.run = run or self.load()

    @property
    def run(self) -> int:
        """Get the current run count from the environment variable."""
        return self._run

    @run.setter
    def run(self, value: Optional[int]) -> None:
        """Set the run count in the environment variable."""
        if value is None:  # Check explicitly for None to avoid blocking valid values like 0
            value = IntPrompt.ask(
                f"[{self.space}][#ffffff]Enter the run \
count for '[{self.radiate}]{self.project_name}[/{self.radiate}]"
            )
            assert isinstance(value, int), "Run count must be an integer."
            assert value > 0, "Run count must be greater than 0."
            self._run = value
        else:
            self._run = value

    def __add__(self, other: int) -> int:
        """Add the run count to another integer."""
        if isinstance(other, int):
            return self.run + other
        raise TypeError("Can only add integers to the run count")

    def load(self, user_run: Optional[int] = None) -> int:
        """Load the current run count from the run file and set it to an environment variable.

        Returns:
            int: The current run count.
        """
        if user_run is not None:
            return user_run
        if self.run_file.exists():
            try:
                # Load from the file
                with open(self.run_file, "r", encoding="utf-8") as file:
                    run_count = int(file.read().strip())
            except ValueError:  # Removed redundant FileNotFoundError
                run_count = 0
        else:
            # File doesn't exist, initialize it and notify the user
            run_count = 0
            console.print(
                f"[{self.space}][i white]Run file '[/i white][b {self.radiate}]{self.run_file}[/b \
{self.radiate}][i white]' does not exist. Creating it with an initial count of [/i white]\
[b {self.radiate}]0[/b {self.radiate}]."
            )
            self.save(run_count)

        # Set the run count in the environment variable
        os.environ[self.env_var_name] = str(run_count)

        return run_count

    def increment(self) -> None:
        """Increment the run count, save it to both the run file and the environment variable."""
        self.run = self.run + 1
        self.save(self.run)

    def reset(self) -> None:
        """Reset the run count to 0 and save it to both the run file and the environment variable."""
        self.run = 0
        self.save(self.run)

    def save(self, run_count: int) -> None:
        """Save the run count to the run file and set it in the environment variable.

        Args:
            run_count (int): The run count to be saved.
        """
        # Save to the file
        try:
            with open(self.run_file, "w", encoding="utf-8") as file:
                file.write(str(run_count))
        except PermissionError as e:
            console.print(f"[on red]PermissionError: {e}[/]")

        # Set the run count in the environment variable
        os.environ[self.env_var_name] = str(run_count)
        if self.verbose:
            console.print(
                f"[on {self.space}][white]Run count updated to [b {self.radiate}]{run_count}\
[/b {self.radiate}]. Stored in:\n\t- [b {self.radiate}]{self.run_file}[/b {self.radiate}] and\n\t- \
environment variable [b {self.radiate}]{self.env_var_name}[/b {self.radiate}]."
            )

    @classmethod
    def get_project_name(cls) -> str:
        """Get the project name from the pyproject.toml file."""
        try:
            with open("pyproject.toml", "r", encoding="utf-8") as file:
                for line in file:
                    if "name" in line:
                        return line.split(" = ")[1].strip().strip('"')
        except FileNotFoundError:
            console.print(
                f"[on {cls.space}][b][#ffffff]No '[/white][{cls.radiate}][{cls.radiate}]pyproject.toml\
[/{cls.radiate}][#ffffff]' file found. Using `default` as name."
            )
            return "default"

        return Path(__file__).parent.name


# Example usage:
if __name__ == "__main__":
    track = Run(verbose=True)

    console.print(
        f"[on {track.space}][b][#ffffff]Current run count: \
[/#ffffff][{track.radiate}]{track.run}[/{track.radiate}][#ffffff]"
    )

    # Increment the run count
    track.increment()

    # The run count is now also available in the environment
    console.print(
        f"[on {track.space}][b][#ffffff]Environment variable \
'[/#ffffff][{track.radiate}]{track.env_var_name}[/{track.radiate}]\
[#ffffff]': [/#ffffff][{track.radiate}]{os.getenv(track.env_var_name)}\
[/{track.radiate}]"
    )
